#!/usr/bin/env python
# -*- coding: UTF-8 -*-
#
# Copyright 2020-2023 NXP
#
# SPDX-License-Identifier: BSD-3-Clause

"""Console script for pfr."""

import logging
import os
import sys
from typing import Callable, Optional, Tuple, Type, Union

import click
from click_option_group import MutuallyExclusiveOptionGroup, optgroup

from spsdk import pfr
from spsdk.apps.elftosb_utils.sb_31_helper import RootOfTrustInfo
from spsdk.apps.utils import spsdk_logger
from spsdk.apps.utils.common_cli_options import (
    FC,
    CommandsTreeGroupAliasedGetCfgTemplate,
    isp_interfaces,
    spsdk_apps_common_options,
)
from spsdk.apps.utils.utils import MBootInterface, catch_spsdk_error, format_raw_data, get_interface
from spsdk.crypto.loaders import extract_public_keys
from spsdk.mboot import McuBoot
from spsdk.pfr.exceptions import SPSDKError, SPSDKPfrConfigError, SPSDKPfrError
from spsdk.pfr.pfrc import Pfrc
from spsdk.utils.misc import find_file, load_binary, load_configuration, size_fmt, write_file
from spsdk.utils.schema_validator import ConfigTemplate

PFRArea = Union[Type[pfr.CMPA], Type[pfr.CFPA]]
logger = logging.getLogger(__name__)


def _store_output(
    data: Union[str, bytes], path: Optional[str], mode: str = "w", msg: Optional[str] = None
) -> None:
    """Store the output data; either on stdout or into file if it's provided."""
    if msg:
        click.echo(msg)
    if path is None:
        click.echo(data)
    else:
        click.echo(f"Result has been stored in: {path}")
        write_file(data, path=path, mode=mode)


def _get_pfr_class(area_name: str) -> PFRArea:
    """Return CMPA/CFPA class based on the name."""
    return getattr(pfr, area_name.upper())


def pfr_device_type_options(no_type: bool = False) -> Callable:
    """PFR common device click options.

    :param no_type: If true, the type option is not added, defaults to False
    :return: Click decorator
    """

    def decorator(options: Callable[[FC], FC]) -> Callable[[FC], FC]:
        """Setup PFR options for device, revision and area (PFR page type)."""
        options = click.option(
            "-r",
            "--revision",
            help="Chip revision; if not specified, most recent one will be used",
        )(options)
        options = click.option(
            "-d",
            "--device",
            type=click.Choice(pfr.CMPA.devices()),
            help="Device to use",
            required=True,
        )(options)
        if not no_type:
            options = click.option(
                "-t",
                "--type",
                "area",
                required=True,
                type=click.Choice(["cmpa", "cfpa"]),
                help="Select PFR partition",
            )(options)
        return options

    return decorator


@click.group(name="pfr", no_args_is_help=True, cls=CommandsTreeGroupAliasedGetCfgTemplate)
@spsdk_apps_common_options
def main(log_level: int) -> int:
    """Utility for generating and parsing Protected Flash Region data (CMPA, CFPA)."""
    spsdk_logger.install(level=log_level)
    return 0


@main.command(name="get-template", no_args_is_help=True)
@pfr_device_type_options()
@click.option(
    "-o",
    "--output",
    type=click.Path(resolve_path=True),
    required=False,
    help="Save the output into a file instead of console",
)
@click.option("-f", "--full", is_flag=True, help="Show full config, including computed values")
def get_template(device: str, revision: str, area: str, output: str, full: bool) -> None:
    """Generate user configuration template file."""
    pfr_obj = _get_pfr_class(area)(device=device, revision=revision)
    data = pfr_obj.get_yaml_config(not full)
    yaml_data = ConfigTemplate.convert_cm_to_yaml(data)
    _store_output(yaml_data, output, msg=f"PFR {area} configuration template has been created.")


@main.command(name="parse-binary", no_args_is_help=True)
@pfr_device_type_options()
@click.option(
    "-o",
    "--output",
    type=click.Path(dir_okay=False),
    required=False,
    help="Save the output into a file instead of console",
)
@click.option(
    "-b",
    "--binary",
    type=click.Path(exists=True, dir_okay=False, resolve_path=True),
    required=True,
    help="Binary to parse",
)
@click.option(
    "-f",
    "--show-diff",
    is_flag=True,
    help="Show differences comparing to defaults",
)
@click.option(
    "-c",
    "--show-calc",
    is_flag=True,
    help="Show also calculated fields when displaying difference to " "defaults (--show-diff)",
)
def parse_binary(
    device: str,
    revision: str,
    area: str,
    output: str,
    binary: str,
    show_calc: bool,
    show_diff: bool,
) -> None:
    """Parse binary and extract configuration."""
    data = load_binary(binary)
    yaml_data = _parse_binary_data(
        data=data,
        device=device,
        revision=revision,
        area=area,
        show_calc=show_calc,
        show_diff=show_diff,
    )
    _store_output(yaml_data, output, msg=f"Success. (PFR: {binary} has been parsed.")


def _parse_binary_data(
    data: bytes,
    device: str,
    area: str,
    revision: Optional[str] = None,
    show_calc: bool = False,
    show_diff: bool = False,
) -> str:
    """Parse binary data and extract YAML configuration.

    :param data: Data to parse
    :param device: Device to use
    :param revision: Revision to use, defaults to 'latest'
    :param area: PFR are (CMPA, CFPA)
    :param show_calc: Also show calculated fields
    :param show_diff: Show only difference to default
    :return: PFR YAML configuration as a string
    """
    pfr_obj = _get_pfr_class(area)(device=device, revision=revision)
    pfr_obj.parse(data)
    parsed = pfr_obj.get_yaml_config(exclude_computed=not show_calc, diff=show_diff)
    yaml_data = ConfigTemplate.convert_cm_to_yaml(parsed)
    return yaml_data


@main.command(name="generate-binary", no_args_is_help=True)
@optgroup.group("Root Of Trust Configuration", cls=MutuallyExclusiveOptionGroup)
@optgroup.option(
    "-e",
    "--elf2sb-config",
    type=click.Path(exists=True, dir_okay=False),
    help="Specify Root Of Trust from configuration file used by elf2sb tool",
)
@optgroup.option(
    "-f",
    "--secret-file",
    type=click.Path(exists=True, dir_okay=False),
    multiple=True,
    help="Secret file (certificate, public key, private key); can be defined multiple times",
)
@click.option(
    "-c",
    "--user-config",
    "user_config_file",
    type=click.Path(exists=True, dir_okay=False),
    required=True,
    help="YAML/JSON file with user configuration",
)
@click.option(
    "-o",
    "--output",
    type=click.Path(dir_okay=False, resolve_path=True),
    required=True,
    help="Save the output into a file instead of console",
)
@click.option("-a", "--add-seal", is_flag=True, help="Add seal mark digest at the end.")
@click.option(
    "-i",
    "--calc-inverse",
    is_flag=True,
    help="Calculate the INVERSE values CAUTION!!! It locks the settings",
)
@click.option(
    "-p",
    "--password",
    help="Password when using Encrypted private keys as --secret-file",
)
@click.option(
    "-x",
    "--force",
    is_flag=True,
    default=False,
    help="Force to generate binary even the config validation fails.",
)
def generate_binary(
    output: str,
    user_config_file: str,
    add_seal: bool,
    calc_inverse: bool,
    elf2sb_config: str,
    secret_file: Tuple[str],
    password: str,
    force: bool,
) -> None:
    """Generate binary data."""
    pfr_config = pfr.PfrConfiguration(user_config_file)
    invalid_reason = pfr_config.is_invalid()
    if invalid_reason:
        raise SPSDKPfrConfigError(
            f"The configuration file is not valid. The reason is: {invalid_reason}"
        )
    area = pfr_config.type
    assert area
    pfr_obj = _get_pfr_class(area)(device=pfr_config.device, revision=pfr_config.revision)
    pfr_obj.set_config(pfr_config, raw=not calc_inverse)
    if pfr_config.device in Pfrc.get_supported_families():
        try:
            pfrc = Pfrc(
                cmpa=pfr_obj if area.lower() == "cmpa" else None,  # type: ignore
                cfpa=pfr_obj if area.lower() == "cfpa" else None,  # type: ignore
            )
            rules = pfrc.validate_brick_conditions()
        except (SPSDKPfrConfigError, SPSDKPfrError) as e:
            logger.debug(f"PFRC unexpectedly failed: {e}")
        else:
            log_text = f"PFRC results: passed: {len(rules[0])}, failed: {len(rules[1])}, ignored: {len(rules[2])}"
            if rules[1]:
                if force:
                    logger.warning(log_text)
                else:
                    raise SPSDKError(log_text)
            else:
                logger.debug(log_text)
    root_of_trust = None
    keys = None
    if elf2sb_config:
        elf2sb_config_dir = os.path.dirname(elf2sb_config)
        public_keys = RootOfTrustInfo(
            load_configuration(elf2sb_config), search_paths=[elf2sb_config_dir]
        ).public_keys
        root_of_trust = tuple((find_file(x, search_paths=[elf2sb_config_dir]) for x in public_keys))
    if secret_file:
        root_of_trust = secret_file
    if area.lower() == "cmpa" and root_of_trust:
        keys = extract_public_keys(root_of_trust, password)
    if not pfr_config.revision:
        pfr_config.revision = pfr_obj.revision
    data = pfr_obj.export(add_seal=add_seal, keys=keys)
    _store_output(data, output, "wb", msg="Success. (PFR binary has been generated)")


@main.command(name="info", no_args_is_help=True)
@pfr_device_type_options()
@click.option(
    "-o",
    "--output",
    type=click.Path(dir_okay=False, resolve_path=True),
    required=True,
    help="Save the output into a file instead of console",
)
@click.option(
    "-p",
    "--open",
    "open_result",
    is_flag=True,
    help="Open the generated description file",
)
def info(device: str, revision: str, area: str, output: str, open_result: bool) -> None:
    """Generate HTML page with brief description of CMPA/CFPA configuration fields."""
    pfr_obj = _get_pfr_class(area)(device=device, revision=revision)
    html_output = pfr_obj.registers.generate_html(
        f"{device.upper()} - {area.upper()}",
        pfr_obj.DESCRIPTION,
        regs_exclude=["SHA256_DIGEST"],
        fields_exclude=["FIELD"],
    )
    _store_output(html_output, output, msg="Success. (PFR info HTML page has been generated)")
    if open_result:  # pragma: no cover # can't test opening the html document
        click.launch(f"{output}")


@main.command(name="devices", no_args_is_help=False)
def devices() -> None:
    """List supported devices."""
    click.echo("\n".join(pfr.CMPA.devices()))


@main.command(name="write", no_args_is_help=True)
@isp_interfaces(
    uart=True,
    usb=True,
    lpcusbsio=True,
    buspal=True,
    json_option=False,
    use_long_timeout_option=True,
)
@pfr_device_type_options()
@click.option(
    "-b",
    "--binary",
    type=click.Path(exists=True, dir_okay=False),
    required=True,
    help="Path to PFR data to write.",
)
def write(
    port: str,
    usb: str,
    buspal: str,
    lpcusbsio: str,
    timeout: int,
    device: str,
    revision: str,
    area: str,
    binary: str,
) -> None:
    """Write PFR page to the device."""
    pfr_obj = _get_pfr_class(area)(device=device, revision=revision)
    pfr_page_address = pfr_obj.config.get_address(device)
    pfr_page_length = pfr_obj.BINARY_SIZE

    click.echo(f"{pfr_obj.__class__.__name__} page address on {device} is {pfr_page_address:#x}")

    data = load_binary(binary)
    if len(data) != pfr_page_length:
        raise SPSDKError(
            f"PFR page length is {pfr_page_length}. Provided binary has {size_fmt(len(data))}."
        )

    interface = get_interface(
        module="mboot", port=port, usb=usb, buspal=buspal, lpcusbsio=lpcusbsio, timeout=timeout
    )
    assert isinstance(interface, MBootInterface)
    with McuBoot(device=interface) as mboot:
        mboot.write_memory(address=pfr_page_address, data=data)
    click.echo(
        f"{pfr_obj.__class__.__name__} data {'written to device.' if mboot.status_code == 0 else 'write failed!'}"
    )


@main.command(name="read", no_args_is_help=True)
@isp_interfaces(
    uart=True,
    usb=True,
    lpcusbsio=True,
    buspal=True,
    json_option=False,
    use_long_timeout_option=True,
)
@pfr_device_type_options()
@click.option(
    "-o",
    "--output",
    type=click.Path(dir_okay=False, resolve_path=True),
    help="Store PFR data into a file. If not specified hexdump data into stdout.",
)
@click.option(
    "-y",
    "--yaml",
    "yaml_output",
    type=click.Path(dir_okay=False, resolve_path=True),
    help="Parse data read from device into YAML config.",
)
@click.option(
    "-f",
    "--show-diff",
    is_flag=True,
    help="(applicable for parsing) Show differences comparing to defaults",
)
@click.option(
    "-c",
    "--show-calc",
    is_flag=True,
    help="(applicable for parsing) Show also calculated fields when displaying difference to "
    "defaults (--show-diff)",
)
def read(
    port: str,
    usb: str,
    buspal: str,
    lpcusbsio: str,
    timeout: int,
    device: str,
    revision: str,
    area: str,
    output: str,
    yaml_output: str,
    show_diff: bool,
    show_calc: bool,
) -> None:
    """Read PFR page from the device."""
    pfr_obj = _get_pfr_class(area)(device=device, revision=revision)
    pfr_page_address = pfr_obj.config.get_address(device)
    pfr_page_length = pfr_obj.BINARY_SIZE
    pfr_page_name = pfr_obj.__class__.__name__

    click.echo(f"{pfr_page_name} page address on {device} is {pfr_page_address:#x}")

    interface = get_interface(
        module="mboot", port=port, usb=usb, buspal=buspal, lpcusbsio=lpcusbsio, timeout=timeout
    )
    assert isinstance(interface, MBootInterface)
    with McuBoot(device=interface) as mboot:
        data = mboot.read_memory(address=pfr_page_address, length=pfr_page_length)
    if not data:
        raise SPSDKError(f"Unable to read data from address {pfr_page_address:#x}")

    if output:
        write_file(data, output, "wb")
        click.echo(f"{pfr_page_name} data stored to {output}")
    if yaml_output:
        yaml_data = _parse_binary_data(
            data=data,
            device=device,
            revision=revision,
            area=area,
            show_calc=show_calc,
            show_diff=show_diff,
        )
        write_file(yaml_data, yaml_output)
        click.echo(f"Parsed config stored to {yaml_output}")
    if not output and not yaml_output:
        click.echo(format_raw_data(data=data, use_hexdump=True))


@main.command(name="erase-cmpa", no_args_is_help=True)
@isp_interfaces(
    uart=True,
    usb=True,
    lpcusbsio=True,
    buspal=True,
    json_option=False,
    use_long_timeout_option=True,
)
@pfr_device_type_options(no_type=True)
def erase_cmpa(
    port: str,
    usb: str,
    buspal: str,
    lpcusbsio: str,
    timeout: int,
    device: str,
    revision: str,
) -> None:
    """Erase CMPA PFR page in the device if is not sealed."""
    pfr_obj = _get_pfr_class("cmpa")(device=device, revision=revision)
    pfr_page_address = pfr_obj.config.get_address(device)
    pfr_page_length = pfr_obj.BINARY_SIZE

    click.echo(f"CMPA page address on {device} is {pfr_page_address:#x}")

    interface = get_interface(
        module="mboot", port=port, usb=usb, buspal=buspal, lpcusbsio=lpcusbsio, timeout=timeout
    )
    assert isinstance(interface, MBootInterface)
    with McuBoot(device=interface) as mboot:
        mboot.write_memory(address=pfr_page_address, data=bytes(pfr_page_length))
    click.echo(f"CMPA page {'has been erased.' if mboot.status_code == 0 else 'erase failed!'}")


@catch_spsdk_error
def safe_main() -> None:
    """Call the main function."""
    sys.exit(main())  # pylint: disable=no-value-for-parameter


if __name__ == "__main__":
    safe_main()



#!/usr/bin/env python
# -*- coding: UTF-8 -*-
#
# Copyright 2021-2023 NXP
#
# SPDX-License-Identifier: BSD-3-Clause

"""Console script for ifr."""

import logging
import sys
from typing import Optional, Union

import click

from spsdk.apps.utils import spsdk_logger
from spsdk.apps.utils.common_cli_options import (
    FC,
    CommandsTreeGroupAliasedGetCfgTemplate,
    isp_interfaces,
    spsdk_apps_common_options,
)
from spsdk.apps.utils.utils import catch_spsdk_error, format_raw_data, get_interface
from spsdk.exceptions import SPSDKError
from spsdk.mboot import McuBoot
from spsdk.mboot.interfaces.base import MBootInterface
from spsdk.pfr import pfr
from spsdk.pfr.exceptions import SPSDKPfrConfigError
from spsdk.pfr.pfr import ROMCFG
from spsdk.utils.misc import load_binary, size_fmt, write_file
from spsdk.utils.schema_validator import ConfigTemplate

logger = logging.getLogger(__name__)


def _parse_binary_data(
    data: bytes,
    device: str,
    revision: Optional[str] = None,
    show_calc: bool = False,
    show_diff: bool = False,
) -> str:
    """Parse binary data and extract YAML configuration.

    :param data: Data to parse
    :param device: Device to use
    :param revision: Revision to use, defaults to 'latest'
    :param show_calc: Also show calculated fields
    :param show_diff: Show only difference to default
    :return: IFR YAML configuration as a string
    """
    ifr_obj = ROMCFG(device=device, revision=revision)
    ifr_obj.parse(data)
    parsed = ifr_obj.get_yaml_config(exclude_computed=not show_calc, diff=show_diff)
    yaml_data = ConfigTemplate.convert_cm_to_yaml(parsed)
    return yaml_data


def _store_output(
    data: Union[str, bytes], path: Optional[str], mode: str = "w", msg: Optional[str] = None
) -> None:
    """Store the output data; either on stdout or into file if it's provided."""
    if msg:
        click.echo(msg)
    if path is None:
        click.echo(data)
    else:
        click.echo(f"Result has been stored in: {path}")
        write_file(data, path=path, mode=mode)


def ifr_device_type_options(options: FC) -> FC:
    """Setup IFR options for device and revision."""
    options = click.option(
        "-r",
        "--revision",
        help="Chip revision; if not specified, most recent one will be used",
    )(options)
    options = click.option(
        "-d",
        "--device",
        type=click.Choice(ROMCFG.devices()),
        help="Device to use",
        required=True,
    )(options)
    return options


@click.group(name="ifr", no_args_is_help=True, cls=CommandsTreeGroupAliasedGetCfgTemplate)
@spsdk_apps_common_options
def main(log_level: int) -> int:
    """Utility for generating and parsing IFR. Please note that IFR0 ROMCFG region is one-time-programmable only."""
    spsdk_logger.install(level=log_level)
    return 0


@main.command(name="devices", no_args_is_help=False)
def devices() -> None:
    """List supported devices."""
    click.echo("\n".join(ROMCFG.devices()))


@main.command(name="get-template", no_args_is_help=True)
@ifr_device_type_options
@click.option(
    "-o",
    "--output",
    type=click.Path(resolve_path=True),
    required=False,
    help="Save the output into a file instead of console",
)
@click.option("-f", "--full", is_flag=True, help="Show full config, including computed values")
def get_template(device: str, revision: str, output: str, full: bool) -> None:
    """Generate user configuration template file."""
    ifr_obj = ROMCFG(device=device, revision=revision)
    data = ifr_obj.get_yaml_config(not full)
    yaml_data = ConfigTemplate.convert_cm_to_yaml(data)
    _store_output(yaml_data, output, msg="IFR configuration template has been created.")


@main.command(name="parse-binary", no_args_is_help=True)
@ifr_device_type_options
@click.option(
    "-o",
    "--output",
    type=click.Path(resolve_path=True),
    required=False,
    help="Save the output into a file instead of console",
)
@click.option(
    "-b",
    "--binary",
    type=click.Path(exists=True, resolve_path=True),
    required=True,
    help="Binary to parse",
)
@click.option("-f", "--show-diff", is_flag=True, help="Show differences comparing to defaults")
def parse_binary(revision: str, output: str, binary: str, show_diff: bool, device: str) -> None:
    """Parse binary and extract configuration."""
    ifr_obj = ROMCFG(device=device, revision=revision)
    data = load_binary(binary)
    ifr_obj.parse(data)
    parsed = ifr_obj.get_yaml_config(exclude_computed=False, diff=show_diff)
    yaml_data = ConfigTemplate.convert_cm_to_yaml(parsed)
    _store_output(yaml_data, output, msg=f"Success. (IFR: {binary} has been parsed.")


@main.command(name="generate-binary", no_args_is_help=True)
@click.option(
    "-d",
    "--device",
    type=click.Choice(ROMCFG.devices()),
    help="Device to use",
    required=True,
)
@click.option(
    "-c",
    "--user-config",
    "user_config_file",
    type=click.Path(exists=True, resolve_path=True),
    required=True,
    help="YAML/JSON file with user configuration",
)
@click.option(
    "-o",
    "--output",
    type=click.Path(resolve_path=True),
    required=True,
    help="Save the output into a file instead of console",
)
def generate_binary(output: str, user_config_file: str, device: str) -> None:
    """Generate binary data."""
    ifr_config = pfr.PfrConfiguration(str(user_config_file))
    invalid_reason = ifr_config.is_invalid()
    if invalid_reason:
        raise SPSDKPfrConfigError(
            f"The configuration file is not valid. The reason is: {invalid_reason}"
        )
    assert ifr_config.type

    ifr_obj = ROMCFG(device=device, revision=ifr_config.revision)
    if not ifr_config.revision:
        ifr_config.revision = ifr_obj.revision
    ifr_obj.set_config(ifr_config, raw=False)

    data = ifr_obj.export()
    _store_output(data, output, "wb", msg="Success. (IFR binary has been generated)")


@main.command(name="write", no_args_is_help=True)
@isp_interfaces(
    uart=True,
    usb=True,
    lpcusbsio=True,
    buspal=True,
    json_option=False,
    use_long_timeout_option=True,
)
@ifr_device_type_options
@click.option(
    "-b",
    "--binary",
    type=click.Path(exists=True, dir_okay=False, resolve_path=True),
    required=True,
    help="Path to IFR data to write.",
)
def write(
    port: str,
    usb: str,
    buspal: str,
    lpcusbsio: str,
    timeout: int,
    device: str,
    revision: str,
    binary: str,
) -> None:
    """Write IFR page to the device."""
    ifr_obj = ROMCFG(device=device, revision=revision)
    ifr_page_address = ifr_obj.config.get_address(device)
    ifr_page_length = ifr_obj.BINARY_SIZE

    click.echo(f"{ifr_obj.__class__.__name__} page address on {device} is {ifr_page_address:#x}")

    data = load_binary(binary)
    if len(data) != ifr_page_length:
        raise SPSDKError(
            f"IFR page length is {ifr_page_length}. Provided binary has {size_fmt(len(data))}."
        )

    interface = get_interface(
        module="mboot", port=port, usb=usb, buspal=buspal, lpcusbsio=lpcusbsio, timeout=timeout
    )
    assert isinstance(interface, MBootInterface)
    with McuBoot(device=interface) as mboot:
        mboot.write_memory(address=ifr_page_address, data=data)
    click.echo(
        f"{ifr_obj.__class__.__name__} data {'written to device.' if mboot.status_code == 0 else 'write failed!'}"
    )


@main.command(name="read", no_args_is_help=True)
@isp_interfaces(
    uart=True,
    usb=True,
    lpcusbsio=True,
    buspal=True,
    json_option=False,
    use_long_timeout_option=True,
)
@ifr_device_type_options
@click.option(
    "-o",
    "--output",
    type=click.Path(dir_okay=False, resolve_path=True),
    help="Store IFR data into a file. If not specified hexdump data into stdout.",
)
@click.option(
    "-y",
    "--yaml",
    "yaml_output",
    type=click.Path(dir_okay=False, resolve_path=True),
    help="Parse data read from device into YAML config.",
)
@click.option(
    "-f",
    "--show-diff",
    is_flag=True,
    help="(applicable for parsing) Show differences comparing to defaults",
)
@click.option(
    "-c",
    "--show-calc",
    is_flag=True,
    help="(applicable for parsing) Show also calculated fields when displaying difference to "
    "defaults (--show-diff)",
)
def read(
    port: str,
    usb: str,
    buspal: str,
    lpcusbsio: str,
    timeout: int,
    device: str,
    revision: str,
    output: str,
    yaml_output: str,
    show_diff: bool,
    show_calc: bool,
) -> None:
    """Read IFR page from the device."""
    ifr_obj = ROMCFG(device=device, revision=revision)
    ifr_page_address = ifr_obj.config.get_address(device)
    ifr_page_length = ifr_obj.BINARY_SIZE
    ifr_page_name = ifr_obj.__class__.__name__

    click.echo(f"{ifr_page_name} page address on {device} is {ifr_page_address:#x}")

    interface = get_interface(
        module="mboot", port=port, usb=usb, buspal=buspal, lpcusbsio=lpcusbsio, timeout=timeout
    )
    assert isinstance(interface, MBootInterface)
    with McuBoot(device=interface) as mboot:
        data = mboot.read_memory(address=ifr_page_address, length=ifr_page_length)
    if not data:
        raise SPSDKError(f"Unable to read data from address {ifr_page_address:#x}")

    if output:
        write_file(data, output, "wb")
        click.echo(f"{ifr_page_name} data stored to {output}")
    if yaml_output:
        yaml_data = _parse_binary_data(
            data=data,
            device=device,
            revision=revision,
            show_calc=show_calc,
            show_diff=show_diff,
        )
        write_file(yaml_data, yaml_output)
        click.echo(f"Parsed config stored to {yaml_output}")
    if not output and not yaml_output:
        click.echo(format_raw_data(data=data, use_hexdump=True))


@catch_spsdk_error
def safe_main() -> None:
    """Call the main function."""
    sys.exit(main())  # pragma: no cover  # pylint: disable=no-value-for-parameter


if __name__ == "__main__":
    safe_main()  # pragma: no cover



#!/usr/bin/env python
# -*- coding: UTF-8 -*-
#
# Copyright (c) 2019-2023 NXP
#
# SPDX-License-Identifier: BSD-3-Clause

"""Module implementing the Mboot communication protocol."""

from .base import Interface, MBootInterface
from .buspal_i2c import scan_buspal_i2c
from .buspal_spi import scan_buspal_spi
from .sdio import Sdio, scan_sdio
from .uart import Uart, scan_uart
from .usb import RawHid, scan_usb
from .usbsio import UsbSio, UsbSioI2C, UsbSioSPI, scan_usbsio, scan_usbsio_i2c, scan_usbsio_spi



#!/usr/bin/env python
# -*- coding: UTF-8 -*-
#
# Copyright 2020-2023 NXP
#
# SPDX-License-Identifier: BSD-3-Clause

"""Utility allowing parsing of HAB audit log data."""

import os
from enum import Enum as PyEnum
from struct import unpack_from
from typing import List, Optional, Type

from spsdk import SPSDKError
from spsdk.mboot import McuBoot, PropertyTag
from spsdk.utils.easy_enum import Enum
from spsdk.utils.misc import load_binary

from .commands import parse_command
from .header import CmdTag

# NOTE HAB Audit log executables have been moved to spsdk/examples/data/hab_audit


# pylint: disable=too-few-public-methods
class CpuInfo:
    """Cpu specific information, necessary for hw tests (HAB log reading)."""

    def __init__(
        self,
        cpu_name: str,
        hab_audit_bin: str,
        bin_base_address: int,
        bin_start_address: int,
    ):
        """Constructor of CpuInfo class."""
        # name of the supported cpus (for example rt1020, rt1050,...)
        self.cpu_name = cpu_name
        # name of the hab audit executable file
        self.hab_audit_bin = hab_audit_bin
        # base address of the hab audit executable file
        self.bin_base_address = bin_base_address
        # start address of the hab audit executable file
        self.bin_start_address = bin_start_address


# pylint: disable=no-member
class CpuData(PyEnum):
    """Data for all supported cpus."""

    MIMXRT1020 = CpuInfo(
        cpu_name="rt1020",
        hab_audit_bin="rt1020_exec_hab_audit.bin",
        bin_base_address=0x20200000,
        bin_start_address=0x20200358,
    )

    MIMXRT1050 = CpuInfo(
        cpu_name="rt1050",
        hab_audit_bin="rt1050_exec_hab_audit.bin",
        bin_base_address=0x20018000,
        bin_start_address=0x20018378,
    )

    MIMXRT1060 = CpuInfo(
        cpu_name="rt1060",
        hab_audit_bin="rt1060_exec_hab_audit.bin",
        bin_base_address=0x20018000,
        bin_start_address=0x200183A4,
    )

    @property
    def cpu_name(self) -> str:
        """:return: name of the cpu."""
        return self.value.cpu_name

    @property
    def bin(self) -> str:
        """:return: name of the hab audit binary."""
        return self.value.hab_audit_bin

    @property
    def base_address(self) -> int:
        """:return: base address of the hab audit bin."""
        return self.value.bin_base_address

    @property
    def start_address(self) -> int:
        """:return: start address of the hab audit bin."""
        return self.value.bin_start_address


class HabConfig(Enum):
    """HAB configurations."""

    HAB_CFG_RETURN = (0x33, "Field Return IC")
    HAB_CFG_OPEN = (0xF0, "Non - secure IC")
    HAB_CFG_CLOSED = (0xCC, "Secure IC")


class HabState(Enum):
    """HAB state definitions."""

    HAB_STATE_INITIAL = (0x33, "Initializing state(transitory)")
    HAB_STATE_CHECK = (0x55, "Check state(non - secure)")
    HAB_STATE_NONSECURE = (0x66, "Non - secure state")
    HAB_STATE_TRUSTED = (0x99, "Trusted state")
    HAB_STATE_SECURE = (0xAA, "Secure state")
    HAB_STATE_FAIL_SOFT = (0xCC, "Soft fail state")
    HAB_STATE_FAIL_HARD = (0xFF, "Hard fail state(terminal)")
    HAB_STATE_NONE = (0xF0, "No security state machine")


class HabStatus(Enum):
    """HAB Status."""

    HAB_STS_ANY = (0, "Match any status")
    HAB_FAILURE = (0x33, "Operation failed")
    HAB_WARNING = (0x69, "Operation completed with warning")
    HAB_SUCCESS = (0xF0, "Operation completed successfully")


class HabReason(Enum):
    """Further reason of the status."""

    HAB_RSN_ANY = (0x00, "Match any reason")
    HAB_ENG_FAIL = (0x30, "Engine failure")
    HAB_INV_ADDRESS = (0x22, "Invalid address: access denied")
    HAB_INV_ASSERTION = (0x0C, "Invalid assertion")
    HAB_INV_CALL = (0x28, "Function called out of sequence")
    HAB_INV_CERTIFICATE = (0x21, "Invalid certificate")
    HAB_INV_COMMAND = (0x06, "Invalid command: command malformed")
    HAB_INV_CSF = (0x11, "Invalid Command Sequence File")
    HAB_INV_DCD = (0x27, "Invalid Device Configuration Data")
    HAB_INV_INDEX = (0x0F, "Invalid index: access denied")
    HAB_INV_IVT = (0x05, "Invalid Image Vector Table")
    HAB_INV_KEY = (0x1D, "Invalid key")
    HAB_INV_RETURN = (0x1E, "Failed callback function")
    HAB_INV_SIGNATURE = (0x18, "Invalid signature")
    HAB_INV_SIZE = (0x17, "Invalid data size")
    HAB_MEM_FAIL = (0x2E, "Memory failure")
    HAB_OVR_COUNT = (0x2B, "Expired poll count")
    HAB_OVR_STORAGE = (0x2D, "Exhausted storage region")
    HAB_UNS_ALGORITHM = (0x12, "Unsupported algorithm")
    HAB_UNS_COMMAND = (0x03, "Unsupported command")
    HAB_UNS_ENGINE = (0x0A, "Unsupported engine")
    HAB_UNS_ITEM = (0x24, "Unsupported configuration item")
    HAB_UNS_KEY = (0x1B, "Unsupported key type or parameters")
    HAB_UNS_PROTOCOL = (0x14, "Unsupported protocol")
    HAB_UNS_STATE = (0x09, "Unsuitable state")


class HabContext(Enum):
    """Context from which the event is logged."""

    HAB_CTX_ANY = (0x00, "Match any context")
    HAB_CTX_ENTRY = (0xE1, "Event logged in hab_rvt.entry()")
    HAB_CTX_TARGET = (0x33, "Event logged in hab_rvt.check_target()")
    HAB_CTX_AUTHENTICATE = (0x0A, "Event logged in hab_rvt.authenticate_image()")
    HAB_CTX_DCD = (0xDD, "Event logged in hab_rvt.run_dcd()")
    HAB_CTX_CSF = (0xCF, "Event logged in hab_rvt.run_csf()")
    HAB_CTX_COMMAND = (0xC0, "Event logged executing CSF or DCD command")
    HAB_CTX_AUT_DAT = (0xDB, "Authenticated data block")
    HAB_CTX_ASSERT = (0xA0, "Event logged in hab_rvt.assert()")
    HAB_CTX_EXIT = (0xEE, "Event logged in hab_rvt.exit()")


class HabEngine(Enum):
    """Engine associated with the failure, or HAB_ENG_ANY if none."""

    HAB_ENG_ANY = (0x00, "Any engine")
    HAB_ENG_SCC = (0x03, "Security controller")
    HAB_ENG_RTIC = (0x05, "Run-time integrity checker")
    HAB_ENG_SAHARA = (0x06, "Crypto accelerator")
    HAB_ENG_CSU = (0x0A, "Central Security Unit")
    HAB_ENG_SRTC = (0x0C, "Secure clock")
    HAB_ENG_DCP = (0x1B, "Data Co-Processor")
    HAB_ENG_CAAM = (0x1D, "Cryptographic Acceleration and Assurance Module")
    HAB_ENG_SNVS = (0x1E, "Secure Non-Volatile Storage")
    HAB_ENG_OCOTP = (0x21, "Fuse controller")
    HAB_ENG_DTCP = (0x22, "DTCP co-processor")
    HAB_ENG_ROM = (0x36, "Protected ROM area")
    HAB_ENG_HDCP = (0x24, "HDCP co-processor")
    HAB_ENG_SW = (0xFF, "Software engine")


def check_reserved_regions(log_addr: int, reserved_regions: Optional[list] = None) -> bool:
    """Checks if the address of the log is not in conflict with CPU reserved regions.

    :param log_addr: address of the RAM, where we want to store hab log
    :param reserved_regions: list with reserved regions
    :return: True if the address of the log is not in conflict, otherwise return False
    """
    if reserved_regions is None:
        return True

    while not len(reserved_regions) % 2 and len(reserved_regions) != 0:
        # region_min and region_max determine one reserved region <region_min,region_max>
        region_max = reserved_regions.pop()
        region_min = reserved_regions.pop()
        # check conflict
        if region_min <= log_addr <= region_max:
            print(
                f"Conflict log address: - [ {hex(log_addr)} ] in region:"
                f" {hex(region_min)} - {hex(region_max)}"
            )
            return False
    return True


def get_hab_log_info(hab_log: Optional[bytes]) -> bool:
    """Gets information about hab log.

    It detects if the hab log is empty, invalid (4x 0xff) or prints out
    valid hab log status.
    :param hab_log: Log with data to test. It can be None.
    :return: False when flashloader is not accessible or problem with
             hab log occurred, otherwise return True.
    """
    if hab_log is None:
        print("Problem during Hab log reading. Hab log is empty.")
        return False
    if hab_log[0:4] == b"\xFF\xFF\xFF\xFF":
        print("Flash not accessible or application entry out of expected value")
        return False

    # first three bytes are HAB status, config and state
    for line in parse_hab_log(hab_log[0], hab_log[1], hab_log[2], hab_log[4:]):
        print(line)
    return True


def get_hab_enum_description(enum_cls: Type[Enum], value: int) -> str:
    """Converts integer value into description of the enumeration value.

    If the value does not match any value from enumeration, it is reported as `Unknown value`

    :param enum_cls: enumeration class used to convert integer to enum
    :param value: to be converted
    :return: description of the converted value
    """
    if value in enum_cls:
        return enum_cls.desc(enum_cls.from_int(value)) + f"  ({hex(value)})"
    return f"{hex(value)} = Unknown value"


def parse_hab_log(hab_sts: int, hab_cfg: int, hab_state: int, data: bytes) -> List[str]:
    """Parse the HAB audit log.

    :param hab_sts: HAB status; this is result of the HAB function `report_status`
    :param hab_cfg: HAB configuration; this is result of the HAB function `report_status`
    :param hab_state: HAB state; this is result of the HAB function `report_status`
    :param data: HAB log data in binary format; result of the HAB function `report_event`
    :return: list of lines to be displayed, that describes the HAB status and content of the LOG
    :raises SPSDKError: If a record has invalid data length
    """
    result = []
    result.append("=" * 60)
    result.append(f"HAB Status:  {get_hab_enum_description(HabStatus, hab_sts)}")
    result.append(f"HAB Config:  {get_hab_enum_description(HabConfig, hab_cfg)}")
    result.append(f"HAB State :  {get_hab_enum_description(HabState, hab_state)}")
    offset = 0
    while offset + 8 <= len(data):
        result.append("=" * 60)
        # parse header
        (cmd, length, version) = unpack_from(">BHB", data, offset)
        if (cmd != 0xDB) or ((version < 0x40) or (version > 0x43)):
            break
        if (length < 8) or (length > 1024):
            raise SPSDKError("invalid log length")
        # parse data
        (sts, rsn, ctx, eng) = unpack_from("4B", data, offset + 4)
        # print results
        result.append(f"Status:  {get_hab_enum_description(HabStatus, sts)}")
        result.append(f"Reason:  {get_hab_enum_description(HabReason, rsn)}")
        result.append(f"Context: {get_hab_enum_description(HabContext, ctx)}")
        result.append(f"Engine:  {get_hab_enum_description(HabEngine, eng)}")
        if length > 8:
            result.append(f"Data:    {data[offset + 8:offset + length].hex().upper()}")
            try:
                cmd = parse_command(data, offset + 8)
                result.append(f"Cmd :    {CmdTag.desc(cmd.tag)}")
                result.append(cmd.info())
            except ValueError:
                pass

        offset += length
    return result


def hab_audit_xip_app(
    cpu_data: CpuData, mboot: McuBoot, read_log_only: bool, hab_audit_path: str
) -> Optional[bytes]:
    """Authenticate the application in external FLASH.

    The function loads application into RAM and invokes its function, that authenticates the application and read the
    HAB log. Then the HAB log is downloaded and parsed and printed to stdout.
    :param cpu_data: target cpu data
    :param mboot: running flashloader
    :param read_log_only: true to read HAB log without invoking authentication; False to authenticate and read-log
        It is recommended to call the function firstly with parameter `True` and second time with parameter False to
        see the difference.
    :param hab_audit_path: Path to directory with HAB audit log executables
    :return: bytes contains result of the hab log, otherwise returns None when an error occurred
    :raises SPSDKError: When flashloader is not running
    :raises SPSDKError: When given cpu data were not provided
    :raises SPSDKError: When there is invalid address
    :raises SPSDKError: When Log address is in conflict with reserved regions
    :raises SPSDKError: When write memory failed
    :raises SPSDKError: When call failed
    """
    # check if the flashloader is running (not None)
    if not mboot:
        raise SPSDKError("Flashloader is not running")

    # get CPU data dir, hab_audit_base and hab_audit_start
    if not cpu_data:
        raise SPSDKError("Can not read the log, because given cpu data were not provided.")
    cpu_data_bin_dir = cpu_data.bin
    evk_exec_hab_audit_base = cpu_data.base_address
    evk_exec_hab_audit_start = cpu_data.start_address

    exec_hab_audit_path = os.path.join(hab_audit_path, cpu_data_bin_dir)

    if not os.path.isfile(exec_hab_audit_path):
        print("\nHAB logger not supported for the processor")
        return None

    # get executable file, that will be loaded into RAM
    exec_hab_audit_code = load_binary(exec_hab_audit_path)
    # find address of the buffer in RAM, where the HAB LOG will be stored
    log_addr = evk_exec_hab_audit_base + exec_hab_audit_code.find(
        b"\xA5\x5A\x11\x22\x33\x44\x55\x66"
    )
    if log_addr <= evk_exec_hab_audit_base:
        raise SPSDKError("Invalid address")
    # check if the executable binary is in collision with reserved region
    reserved_regions = mboot.get_property(PropertyTag.RESERVED_REGIONS)

    # check conflict between hab log address and any of reserved regions
    # we need 2 values (min and max) - that is why %2 is used
    if not check_reserved_regions(log_addr, reserved_regions):
        raise SPSDKError("Log address is in conflict with reserved regions")
    if not mboot.write_memory(evk_exec_hab_audit_base, exec_hab_audit_code, 0):
        raise SPSDKError("Write memory failed")
    if not mboot.call(evk_exec_hab_audit_start | 1, 0 if read_log_only else 1):
        raise SPSDKError("Call failed")

    log = mboot.read_memory(log_addr, 100, 0)
    return log



#!/usr/bin/env python
# -*- coding: UTF-8 -*-
#
# Copyright 2017-2018 Martin Olejar
# Copyright 2019-2021 NXP
#
# SPDX-License-Identifier: BSD-3-Clause

"""Error codes defined by the SDP protocol."""

from spsdk.utils.easy_enum import Enum


########################################################################################################################
# SDP Status Codes (Errors)
########################################################################################################################
class StatusCode(Enum):
    """SDP status codes."""

    SUCCESS = (0, "Success", "Success")
    CMD_FAILURE = (1, "CommandFailure", "Command Failure")
    HAB_IS_LOCKED = (2, "HabIsLocked", "HAB Is Locked")
    READ_DATA_FAILURE = (10, "ReadDataFailure", "Read Register/Data Failure")
    WRITE_REGISTER_FAILURE = (11, "WriteRegisterFailure", "Write Register Failure")
    WRITE_IMAGE_FAILURE = (12, "WriteImageFailure", "Write Image Failure")
    WRITE_DCD_FAILURE = (13, "WriteDcdFailure", "Write DCD Failure")
    WRITE_CSF_FAILURE = (14, "WriteCsfFailure", "Write CSF Failure")
    SKIP_DCD_HEADER_FAILURE = (15, "SkipDcdHeaderFailure", "Skip DCD Header Failure")


########################################################################################################################
# HAB Status Codes (Errors)
########################################################################################################################
class HabStatusInfo(Enum):
    """HAB status codes."""

    UNKNOWN = (0x00, "Unknown")
    WARNING = (0x69, "Warning")
    ERROR = (0x33, "Failure")
    SUCCESS = (0xF0, "Success")


class HabErrorReason(Enum):
    """HAB Error Reason."""

    UNKNOWN = (0x00, "Unknown Reason")
    ENGINE_FAILURE = (0x30, "Engine Failure")
    INVALID_ADDRESS = (0x22, "Invalid Address: Access Denied")
    INVALID_ASSERTION = (0x0C, "Invalid Assertion")
    INVALID_CERTIFICATE = (0x21, "Invalid Certificate")
    INVALID_COMMAND = (0x06, "Invalid Command: Malformed")
    INVALID_CSF = (0x11, "Invalid CSF")
    INVALID_DCD = (0x27, "Invalid DCD")
    INVALID_IVT = (0x05, "Invalid IVT")
    INVALID_KEY = (0x1D, "Invalid Key")
    INVALID_MAC = (0x32, "Invalid MAC")
    INVALID_BLOB = (0x31, "Invalid Blob")
    INVALID_INDEX = (0x0F, "Invalid Index: Access Denied")
    FAILED_CALLBACK = (0x1E, "Failed Callback Function")
    INVALID_SIGNATURE = (0x18, "Invalid Signature")
    INVALID_DATA_SIZE = (0x17, "Invalid Data Size")
    MEMORY_FAILURE = (0x2E, "Memory Failure")
    CALL_OUT_OF_SEQUENCE = (0x28, "Function Called Out Of Sequence")
    EXPIRED_POLL_COUNT = (0x2B, "Expired Poll Count")
    EXHAUSTED_STORAGE_REGION = (0x2D, "Exhausted Storage Region")
    UNSUPPORTED_ALGORITHM = (0x12, "Unsupported Algorithm")
    UNSUPPORTED_COMMAND = (0x03, "Unsupported Command")
    UNSUPPORTED_ENGINE = (0x0A, "Unsupported Engine")
    UNSUPPORTED_CONF_ITEM = (0x24, "Unsupported Configuration Item")
    UNSUPPORTED_KEY_OR_PARAM = (0x1B, "Unsupported Key Type or Parameters")
    UNSUPPORTED_PROTOCOL = (0x14, "Unsupported Protocol")
    UNSUITABLE_STATE = (0x09, "Unsuitable State")


class HabErrorContext(Enum):
    """HAB Error Context."""

    HAB_CTX_ANY = (0x00, "Match any context in hab_rvt.report_event()")
    HAB_FAB_TEST = (0xFF, "Event logged in hab_fab_test()")
    HAB_RVT_ENTRY = (0xE1, "Event logged in hab_rvt.entry()")
    RVT_CHECK_TARGET = (0x33, "Event logged in hab_rvt.check_target()")
    RVT_AUTHENTICATE_IMG = (0x0A, "Event logged in hab_rvt.authenticate_image()")
    RVT_RUN_DCD = (0xDD, "Event logged in hab_rvt.run_dcd()")
    RVT_RUN_CSF = (0xCF, "Event logged in hab_rvt.run_csf()")
    RVT_CSF_DCD_CMD = (0xC0, "Event logged executing CSF or DCD command")
    RVT_ASSERT = (0xA0, "Event logged in hab_rvt.assert()")
    RVT_EXIT = (0xEE, "Event logged in hab_rvt.exit()")
    AUTH_DATA_BLOCK = (0xDB, "Authenticated data block")



#!/usr/bin/env python
# -*- coding: UTF-8 -*-
#
# Copyright 2022-2023 NXP
#
# SPDX-License-Identifier: BSD-3-Clause
"""DK6 Device high level API."""
import logging
from types import TracebackType
from typing import Callable, Dict, List, Optional, Type, Union

from spsdk import SPSDKError
from spsdk.utils.easy_enum import Enum

from .commands import (
    GetChipIdResponse,
    MemGetInfoResponse,
    MemoryAccessValues,
    MemoryId,
    MemoryType,
    StatusCode,
)
from .interface import Uart
from .protocol import DK6Protocol
from .serial_device import SerialDevice

logger = logging.getLogger(__name__)


MAX_MEM_ID = 9

MAC_MEM_ADDR = 0x9FC70
DEV_TYPE_ADDR = 0x9FC60
MAC_LEN = 8
DEV_LEN = 4


class DK6ChipIdInternal(Enum):
    """DK6 Internal chip ID."""

    JN5189 = (0x88888888, "JN5189 ESx")
    QN9090 = (0x1240C686, "QN9090")
    K32W041 = (0x1300C686, "K32W041")
    K32W061 = (0x1440C686, "K32W061")


class DK6DeviceId(Enum):
    """DK6 Device IDs."""

    JN5188 = (5188, "JN5188")
    JN5189 = (5189, "JN5189")
    QN9030 = (9030, "QN9030")
    QN9090 = (9090, "QN9090")
    K32W041 = (32041, "K32W041")
    K32W061 = (32061, "K32W061")
    UNKNOWN = (0, "UNKNOWN")


class DK6Memory:
    """Class that holds information about the DK6 devices memory."""

    def __init__(
        self,
        base_address: int,
        length: int,
        sector_size: int,
        mem_type: MemoryType,
        mem_name: str,
        mem_id: MemoryId,
        access: MemoryAccessValues,
    ) -> None:
        """DK6Memory Constructor.

        :param base_address: Memory base address
        :param length: Memory length
        :param sector_size: Memory sector size
        :param mem_type: Memory type
        :param mem_name: Memory name
        :param mem_id: Memory ID
        :param access: Memory access
        """
        self.base_address = base_address
        self.length = length
        self.sector_size = sector_size
        self.mem_type = mem_type
        self.mem_name = mem_name
        self.mem_id = mem_id
        self.access = access

    def __str__(self) -> str:
        return (
            f"\tMemoryId={MemoryId.get(self.mem_id)}\n"
            f"\tBaseAddress={hex(self.base_address)}\n"
            f"\tLength={hex(self.length)}\n"
            f"\tSectorSize={hex(self.sector_size)}\n"
            f"\tMemoryType={MemoryType.get(self.mem_type)}\n"
            f"\tAccess={MemoryAccessValues.get(self.access)}\n"
        )

    def __repr__(self) -> str:
        """Return obj representation.

        :return: return obj str
        """
        return self.__str__()

    @property
    def end_address(self) -> int:
        """End address of Memory.

        :return: End address
        """
        return self.base_address + self.length


def check_memory(
    memory: DK6Memory,
    access: MemoryAccessValues,
    length: int,
    relative: bool,
    address: int,
) -> int:
    """Check memory range and return sanitized address value.

    :param memory: DK6Memory
    :param access: access type
    :param length: length of data
    :param relative: true if address is relative to base address
    :param address: memory address
    :raises SPSDKError: if memory ID is not supported
    :raises SPSDKError: if access is not allowed
    :raises SPSDKError: if the memory range is invalid
    :return: Sanitized memory address
    """
    if memory is None:
        raise SPSDKError("Memory ID is not supported")

    if access > memory.access:
        raise SPSDKError(f"Access {access} is not allowed. Only allowed is {hex(memory.access)}")

    if relative:
        address += memory.base_address
        mem_check = (address + length) > memory.end_address
    else:
        mem_check = ((address + length) > memory.end_address) or (address < memory.base_address)

    if mem_check:
        raise SPSDKError(
            f"Invalid range. The range is {hex(memory.base_address)}:{hex(memory.end_address)}"
            f" or 0:{hex(memory.length)} in relative mode"
        )

    return address


class DK6Device:
    """Class that represents DK6 device.

    It's a high level class that encapsulates communication interface and protocol
    """

    def __init__(
        self,
        device: SerialDevice,
    ) -> None:
        """DK6Device constructor.

        :param device: SerialDevice that will be used for communication
        :param baudrate: communication baudrate, defaults to 115200
        """
        self.memories: Dict[int, DK6Memory] = {}
        self.chip_id: Union[GetChipIdResponse, None] = None
        self.uart = Uart(device)
        self.protocol = DK6Protocol(self.uart)
        self.mac_addr: Union[None, bytes] = None
        self.dev_type: Union[None, DK6DeviceId] = None
        self.initialized = False

    def __del__(self) -> None:
        logger.info("Closing DK6 device")
        self.close()

    def __enter__(self) -> "DK6Device":
        return self

    def __exit__(
        self,
        exception_type: Optional[Type[BaseException]] = None,
        exception_value: Optional[BaseException] = None,
        traceback: Optional[TracebackType] = None,
    ) -> None:
        logger.info("Closing DK6 device")
        self.close()

    def _split_data(self, data: bytes) -> List[bytes]:
        """Split data to send if necessary.

        :param data: Data to send
        :return: List of data splices
        """
        max_packet_size = self.protocol.MAX_PAYLOAD_SIZE
        return [data[i : i + max_packet_size] for i in range(0, len(data), max_packet_size)]

    def close(self) -> None:
        """Close UART device.

        :raises: SPSDKError: When the device cannot be closed
        """
        if self.uart:
            self.uart.close()
        self.initialized = False

    def add_memory(self, memory: DK6Memory) -> None:
        """Add memory to the list of available memories.

        :param memory: DK6Memory
        """
        self.memories.update({memory.mem_id: memory})

    def get_memory(self, memory_id: MemoryId) -> DK6Memory:
        """Get DK6Memory based on memory_id.

        :param memory_id: MemoryId of the desired memory
        :raises SPSDKError: When the memory cannot be fetched
        :return: DK6Memory
        """
        if self.memories:
            memory = self.memories.get(memory_id)
            if memory:
                return memory
        raise SPSDKError(f"Memory with {memory_id} is not fetched")

    def get_mac_str(self) -> str:
        """Get MAC address in string format.

        :return: string containing MAC address FF:FF..
        """
        if self.mac_addr:
            return ":".join(f"{b:02X}" for b in self.mac_addr)
        else:
            return "N/A"

    def add_memory_from_response(self, memory_response: MemGetInfoResponse) -> None:
        """Add memory from MemGetInfoResponse.

        :param memory_response: MemGetInfoResponse
        """
        if memory_response.status == StatusCode.OK:
            memory = DK6Memory(
                base_address=memory_response.base_addr,
                length=memory_response.length,
                sector_size=memory_response.sector_size,
                mem_type=memory_response.mem_type,
                mem_name=memory_response.mem_name,
                mem_id=memory_response.memory_id,
                access=memory_response.access,
            )
            self.add_memory(memory)

    def init(self) -> None:
        """Initialize DK6 device for communication.

        1. Unlock ISP default
        2. Get device information
        3. Unlock ISP with default key
        4. Get info about memories
        """
        if not self.initialized:
            logger.info("Initializing device... Sending UNLOCK ISP default")
            self.protocol.unlock_isp_default()
            logger.info("Obtaining device information")
            self.chip_id = self.protocol.get_device_information()
            logger.info("Unlocking ISP")
            self.protocol.unlock_isp()

            logger.info("Getting information about memories")
            for mem_id in range(MAX_MEM_ID):
                mem_info = self.protocol.mem_get_info(mem_id)
                self.add_memory_from_response(mem_info)

            self.protocol.mem_close()

            logger.info("Reading MAC address")
            self.protocol.mem_open(MemoryId.Config)
            response = self.protocol.mem_read(MAC_MEM_ADDR, MAC_LEN)
            self.mac_addr = response.data
            self.protocol.mem_close()

            logger.info("Reading device type")
            self.protocol.mem_open(MemoryId.Config)
            response = self.protocol.mem_read(DEV_TYPE_ADDR, DEV_LEN)
            try:
                self.dev_type = DK6DeviceId.from_int(int.from_bytes(response.data, "little"))
            except SPSDKError:
                self.dev_type = DK6DeviceId.UNKNOWN

            self.protocol.mem_close()
            self.initialized = True
        else:
            logger.info("Skipping Initialization, device is already initialized")

    def read_memory(
        self,
        memory_id: MemoryId,
        address: int,
        length: int,
        access: MemoryAccessValues = MemoryAccessValues.WRITE,
        progress_callback: Optional[Callable[[int, int], None]] = None,
        relative: bool = False,
    ) -> bytes:
        """Read memory from the DK6 device.

        1. Make a validation of the read request
        2. Open memory in given access mode
        3. Split read request to chunks of max(MAX_PAYLOAD_SIZE, requested_len)
        4. Read data
        5. Close memory

        :param memory_id: MemoryID of the memory to be used
        :param address: start address
        :param length: length of data
        :param access: memory access value, defaults to MemoryAccessValues.WRITE
        :param progress_callback: progress callback used in CLI, defaults to None
        :param relative: True if address is relative to the memory base address
        :raises SPSDKError: Memory ID is not supported
        :raises SPSDKError: Access is not allowed
        :raises SPSDKError: Invalid range
        :return: Read data
        """
        memory = self.get_memory(memory_id)
        address = check_memory(memory, access, length, relative, address)
        logger.info(f"READ command, memory {memory_id}, address {address}, length {length}")
        self.protocol.mem_open(memory_id, access)

        payload_size = self.protocol.MAX_PAYLOAD_SIZE
        packets = length // payload_size
        remainder = length % payload_size

        if remainder:
            packets += 1

        data = b""

        for idx in range(packets):
            if idx == packets - 1 and remainder:
                data_len = remainder
            else:
                data_len = payload_size
            response = self.protocol.mem_read(address + idx * payload_size, data_len)
            data += response.data
            if progress_callback:
                progress_callback(len(data), length)
        self.protocol.mem_close()

        return data

    def write_memory(
        self,
        memory_id: MemoryId,
        address: int,
        length: int,
        data: bytes,
        access: MemoryAccessValues = MemoryAccessValues.ALL,
        progress_callback: Optional[Callable[[int, int], None]] = None,
        relative: bool = False,
    ) -> None:
        """Write memory to the DK6 device.

        1. Make a validation of the read request
        2. Open memory in given access mode
        3. Split write request to chunks of max(MAX_PAYLOAD_SIZE, requested_len)
        4. Write data
        5. Close memory

        :param memory_id: MemoryID of the memory to be used
        :param address: start address
        :param length: length of data
        :param data: data to be written
        :param access: memory access value, defaults to MemoryAccessValues.WRITE
        :param progress_callback: progress callback used in CLI, defaults to None
        :param relative: True if address is relative to the memory base address
        :raises SPSDKError: Memory ID is not supported
        :raises SPSDKError: Access is not allowed
        :raises SPSDKError: Invalid range
        :raises SPSDKError: No response from device
        """
        memory = self.get_memory(memory_id)

        address = check_memory(memory, access, length, relative, address)

        self.protocol.mem_open(memory_id, access)

        data_chunks = self._split_data(data)

        total_sent = 0
        total_to_send = len(data)

        try:
            for data_chunk in data_chunks:
                status = self.protocol.mem_write(
                    address + total_sent, len(data_chunk), data_chunk
                ).status
                if status != StatusCode.OK:
                    raise SPSDKError("Sending of data failed")
                total_sent += len(data_chunk)
                if progress_callback:
                    progress_callback(total_sent, total_to_send)

        except TimeoutError as exc:
            logger.error("RX: No Response, Timeout Error !")
            raise SPSDKError("No Response from Device") from exc

        self.protocol.mem_close()

    def erase_memory(
        self,
        memory_id: MemoryId,
        address: int,
        length: int,
        access: MemoryAccessValues = MemoryAccessValues.ALL,
        progress_callback: Optional[Callable[[int, int], None]] = None,
        relative: bool = False,
        verify: bool = False,
    ) -> None:
        """Erase memory of DK6 device.

        # 1. Make a validation of the read request
        # 2. Open memory in given access mode
        # 4. Erase data
        # 5. Optionally verify with blank check
        # 6. Close memory

        :param memory_id: MemoryID of the memory to be used
        :param address: start address
        :param length: length of data
        :param access: memory access value, defaults to MemoryAccessValues.WRITE
        :param progress_callback: progress callback used in CLI, defaults to None
        :param relative: True if address is relative to the memory base address
        :param verify: True for erase verification by memory blank check
        :raises SPSDKError: Memory ID is not supported
        :raises SPSDKError: Access is not allowed
        :raises SPSDKError: Invalid range
        :raises SPSDKError: No response from device
        """
        memory = self.get_memory(memory_id)

        if progress_callback:
            progress_callback(1, 4)

        address = check_memory(memory, access, length, relative, address)

        self.protocol.mem_open(memory_id, access)
        result = self.protocol.mem_erase(address, length)
        if result.status != StatusCode.OK:
            raise SPSDKError("Memory erase failed")

        if progress_callback:
            progress_callback(3, 4)
        if verify:
            result_check = self.protocol.mem_blank_check(address, length)
            if result_check.status != StatusCode.OK:
                raise SPSDKError("Memory blank check failed")
        self.protocol.mem_close()

        if progress_callback:
            progress_callback(4, 4)

    def reset(self) -> None:
        """Resets device."""
        result = self.protocol.reset()

        if result.status != StatusCode.OK:
            raise SPSDKError("Reset failed")



#!/usr/bin/env python
# -*- coding: UTF-8 -*-
#
# Copyright 2020-2022 NXP
#
# SPDX-License-Identifier: BSD-3-Clause

"""Commands for SBFile."""

from typing import Mapping, Type

from spsdk import SPSDKError

from ..sb2.commands import (
    CmdBaseClass,
    CmdCall,
    CmdErase,
    CmdFill,
    CmdJump,
    CmdLoad,
    CmdMemEnable,
    CmdNop,
    CmdProg,
    CmdReset,
    CmdTag,
    EnumCmdTag,
)

# mapping of V1.x command to the implementation class
_CMDV1_TO_CLASS: Mapping[EnumCmdTag, Type[CmdBaseClass]] = {
    EnumCmdTag.NOP: CmdNop,
    EnumCmdTag.TAG: CmdTag,
    EnumCmdTag.LOAD: CmdLoad,
    EnumCmdTag.FILL: CmdFill,
    EnumCmdTag.JUMP: CmdJump,
    EnumCmdTag.CALL: CmdCall,
    # RESERVED == 06
    EnumCmdTag.ERASE: CmdErase,
    EnumCmdTag.RESET: CmdReset,
    EnumCmdTag.MEM_ENABLE: CmdMemEnable,
    EnumCmdTag.PROG: CmdProg,
}


def parse_v1_command(data: bytes, offset: int = 0) -> CmdBaseClass:
    """Parse SB V1.x command from binary format.

    :param data: Input data as bytes
    :param offset: The offset of input data to start parsing
    :return: parsed command object
    :raises SPSDKError: Raised when there is unsupported command
    """
    header_tag = EnumCmdTag.from_int(data[offset + 1])
    if header_tag not in _CMDV1_TO_CLASS:
        raise SPSDKError(f"Unsupported command: {EnumCmdTag.name(header_tag)}")
    return _CMDV1_TO_CLASS[header_tag].parse(data, offset)



#!/usr/bin/env python
# -*- coding: UTF-8 -*-
#
# Copyright 2019-2023 NXP
#
# SPDX-License-Identifier: BSD-3-Clause

"""Module for base abstract classes."""

from abc import ABC, abstractmethod
from typing import Any, Optional

from spsdk.crypto import PrivateKey


########################################################################################################################
# Abstract Class for Security Backend
########################################################################################################################
class BackendClass(ABC):
    """Abstract Class for Security Backend."""

    @property
    @abstractmethod
    def name(self) -> str:
        """Backend Name."""

    @property
    @abstractmethod
    def version(self) -> str:
        """Backend Version."""

    @abstractmethod
    def random_bytes(self, length: int) -> bytes:
        """Return a random byte string with specified length.

        :param length: The length in bytes
        """

    @abstractmethod
    def hash(self, data: bytes, algorithm: str = "sha256") -> bytes:
        """Return a HASH from input data with specified algorithm.

        :param data: Input data in bytes
        :param algorithm: Algorithm type for HASH function
        """

    @abstractmethod
    def hmac(self, key: bytes, data: bytes, algorithm: str = "sha256") -> bytes:
        """Return a HMAC from data with specified key and algorithm.

        :param key: The key in bytes format
        :param data: Input data in bytes format
        :param algorithm: Algorithm type for HASH function (sha256, sha384, sha512, ...)
        """

    @abstractmethod
    def aes_key_wrap(self, kek: bytes, key_to_wrap: bytes) -> bytes:
        """Wraps a key using a key-encrypting key (KEK).

        :param kek: The key-encrypting key
        :param key_to_wrap: Plain data
        """

    @abstractmethod
    def aes_key_unwrap(self, kek: bytes, wrapped_key: bytes) -> bytes:
        """Unwraps a key using a key-encrypting key (KEK).

        :param kek: The key-encrypting key
        :param wrapped_key: Encrypted data
        """

    @abstractmethod
    def aes_ctr_encrypt(self, key: bytes, plain_data: bytes, nonce: bytes) -> bytes:
        """Encrypt plain data with AES in CTR mode.

        :param key: The key for data encryption
        :param plain_data: Input data
        :param nonce: Nonce data with counter value
        """

    @abstractmethod
    def aes_ctr_decrypt(self, key: bytes, encrypted_data: bytes, nonce: bytes) -> bytes:
        """Decrypt encrypted data with AES in CTR mode.

        :param key: The key for data decryption
        :param encrypted_data: Input data
        :param nonce: Nonce data with counter value
        """

    @abstractmethod
    def aes_xts_encrypt(self, key: bytes, plain_data: bytes, tweak: bytes) -> bytes:
        """Encrypt plain data with AES in XTS mode.

        :param key: The key for data encryption
        :param plain_data: Input data
        :param tweak: The tweak is a 16 byte value
        """

    @abstractmethod
    def aes_xts_decrypt(self, key: bytes, encrypted_data: bytes, tweak: bytes) -> bytes:
        """Decrypt encrypted data with AES in XTS mode.

        :param key: The key for data decryption
        :param encrypted_data: Input data
        :param tweak: The tweak is a 16 byte value
        """

    @abstractmethod
    def aes_cbc_encrypt(
        self, key: bytes, plain_data: bytes, iv_data: Optional[bytes] = None
    ) -> bytes:
        """Encrypt plain data with AES in CBC mode.

        :param key: The key for data encryption
        :param plain_data: Input data
        :param iv_data: Initialization vector data
        """

    @abstractmethod
    def aes_cbc_decrypt(
        self, key: bytes, encrypted_data: bytes, iv_data: Optional[bytes] = None
    ) -> bytes:
        """Decrypt encrypted data with AES in CBC mode.

        :param key: The key for data decryption
        :param encrypted_data: Input data
        :param iv_data: Initialization vector data
        """

    @abstractmethod
    def rsa_sign(self, private_key: bytes, data: bytes, algorithm: str = "sha256") -> bytes:
        """Sign input data.

        :param private_key: The private key
        :param data: Input data
        :param algorithm: Used algorithm
        """

    @abstractmethod
    def rsa_verify(
        self,
        pub_key_mod: int,
        pub_key_exp: int,
        signature: bytes,
        data: bytes,
        algorithm: str = "sha256",
    ) -> bool:
        """Verify input data.

        :param pub_key_mod: The private key modulus
        :param pub_key_exp: The private key exponent
        :param signature: The signature of input data
        :param data: Input data
        :param algorithm: Used algorithm
        """

    @abstractmethod
    def rsa_public_key(self, modulus: int, exponent: int) -> Any:
        """Create RSA public key object from modulus and exponent.

        :param modulus: The RSA public key modulus
        :param exponent: The RSA public key exponent
        """

    @abstractmethod
    def ecc_sign(self, private_key: bytes, data: bytes, algorithm: Optional[str] = None) -> bytes:
        """Sign data using (EC)DSA.

        :param private_key: ECC private key
        :param data: Data to sign
        :param algorithm: Hash algorithm, if None the hash length is determined from ECC curve size
        :return: Signature, r and s coordinates as bytes
        """

    @abstractmethod
    def ecc_verify(
        self, public_key: bytes, signature: bytes, data: bytes, algorithm: Optional[str] = None
    ) -> bool:
        """Verify (EC)DSA signature.

        :param public_key: ECC public key
        :param signature: Signature to verify, r and s coordinates as bytes
        :param data: Data to validate
        :param algorithm: Hash algorithm, if None the hash length is determined from ECC curve size
        :return: True if the signature is valid
        :raises SPSDKError: Signature length is invalid
        """

    @staticmethod
    @abstractmethod
    def sign_size(key: PrivateKey) -> int:
        """Get size of signature for loaded private key.

        :param key: Private key used to sign data.
        :return: Size of signature in bytes for the private key.
        :raises SPSDKError: Invalid key type.
        """


########################################################################################################################
# Abstract Class for Data Classes
########################################################################################################################
# TODO Refactor: this class should not be part of crypto module
class BaseClass(ABC):
    """Abstract Class for Data Classes."""

    def __eq__(self, obj: Any) -> bool:
        """Check object equality."""
        return isinstance(obj, self.__class__) and vars(obj) == vars(self)

    def __ne__(self, obj: Any) -> bool:
        return not self.__eq__(obj)

    def __str__(self) -> str:
        """Object description in string format."""
        return self.info()

    @abstractmethod
    def info(self) -> str:
        """Object description in string format."""

    @abstractmethod
    def export(self) -> bytes:
        """Serialize object into bytes array."""

    @classmethod
    @abstractmethod
    def parse(cls, data: bytes, offset: int = 0) -> "BaseClass":
        """Deserialize object from bytes array."""



#!/usr/bin/env python
# -*- coding: UTF-8 -*-
#
# Copyright 2020-2023 NXP
#
# SPDX-License-Identifier: BSD-3-Clause

"""Module provides support for TrustZone configuration data."""
import logging
import os
import struct
from typing import Any, Dict, List, Optional

from spsdk import SPSDKError
from spsdk.exceptions import SPSDKValueError
from spsdk.image import TZ_SCH_FILE
from spsdk.utils.database import Database
from spsdk.utils.easy_enum import Enum
from spsdk.utils.misc import format_value, load_configuration, value_to_int
from spsdk.utils.schema_validator import ConfigTemplate, ValidationSchemas

logger = logging.getLogger(__name__)


class TrustZoneType(Enum):
    """Enum defining various types of TrustZone types."""

    ENABLED = (0x0, "TrustZone enabled with default settings")
    CUSTOM = (0x1, "TrustZone enabled with custom settings")
    DISABLED = (0x2, "Disabled")


class TrustZone:
    """Provide creation of binary data to set up the TrustZone engine in CM-33."""

    PRESET_DIR = os.path.normpath(
        os.path.join(os.path.dirname(__file__), "..", "data", "tz_presets")
    )
    CONFIG_FILE = os.path.join(PRESET_DIR, "database.yaml")

    @classmethod
    def enabled(cls) -> "TrustZone":
        """Alternate constructor for ENABLED type of TrustZone."""
        return cls(tz_type=TrustZoneType.ENABLED)

    @classmethod
    def disabled(cls) -> "TrustZone":
        """Alternate constructor for DISABLED type of TrustZone."""
        return cls(tz_type=TrustZoneType.DISABLED)

    @classmethod
    def custom(
        cls, family: str, customizations: dict, revision: Optional[str] = None
    ) -> "TrustZone":
        """Alternate constructor for CUSTOM type of TrustZone."""
        return cls(
            tz_type=TrustZoneType.CUSTOM,
            family=family,
            revision=revision,
            customizations=customizations,
        )

    @classmethod
    def from_binary(
        cls, family: str, raw_data: bytes, revision: Optional[str] = None
    ) -> "TrustZone":
        """Alternate constructor using existing binary data."""
        return cls(
            tz_type=TrustZoneType.CUSTOM,
            family=family,
            revision=revision,
            raw_data=raw_data,
        )

    @classmethod
    def from_config(cls, config_data: Dict[str, Any]) -> "TrustZone":
        """Alternate constructor using configuration data.

        :raises SPSDKError: Invalid configuration file.
        :return: TrustZone class instance.
        """
        try:
            family = config_data["family"]
            revision = config_data.get("revision")
            presets = config_data["trustZonePreset"]
            return cls.custom(family=family, customizations=presets, revision=revision)
        except (TypeError, SPSDKError) as exc:
            raise SPSDKError(f"Invalid TrustZone configuration file: {str(exc)}") from exc

    def __init__(
        self,
        tz_type: TrustZoneType = TrustZoneType.ENABLED,
        family: Optional[str] = None,
        revision: Optional[str] = None,
        customizations: Optional[dict] = None,
        raw_data: Optional[bytes] = None,
    ) -> None:
        """Initialize the trustzone."""
        self.type = tz_type
        self.family = family
        self.database = self.load_database()
        self.customs = customizations
        self.revision = revision

        if self.type == TrustZoneType.DISABLED and customizations:
            raise SPSDKError("TrustZone was disabled, can't add trust_zone_data")

        if self.customs is not None:
            self.type = TrustZoneType.CUSTOM

        if self.type == TrustZoneType.CUSTOM:
            if not self.family:
                raise SPSDKError("Need to provide 'family' parameter")
            self.family = self.family.lower()
            if self.family not in self.get_families():
                raise SPSDKError(f"Chip family '{self.family}' is not supported\n")
            self.revision = self.sanitize_revision(self.family, self.revision)
            if self.revision not in self.get_revisions():
                raise SPSDKError(
                    f"Revision '{self.revision}' is not supported on family '{self.family}'\n"
                )

            self.presets: dict = self._load_presets()
            if raw_data:
                self.customs = self._parse_raw_data(raw_data)
            if self.customs is None:
                raise SPSDKError("Need to provide 'customization' parameter")

            if not TrustZone.validate_custom_data(self.presets, self.customs):
                raise SPSDKError(
                    "Invalid register found in customization data:\n"
                    f"{[item for item in self.customs if item not in self.presets]}"
                )

    @classmethod
    def get_preset_data_size(cls, family: str, revision: str = "latest") -> int:
        """Get size of preset data in binary form.

        :param family: Family description.
        :param revision: Chip revision specification, as default, latest is used.
        :raises SPSDKValueError: Family or revision is not supported.
        :return: Size of TZ data.
        """
        database = cls.load_database()
        file_name = database.devices.get_by_name(family).revisions.get(revision).data_file
        if not file_name:
            raise SPSDKValueError(f"Data file for {family} does't exist!")
        return len(load_configuration(os.path.join(TrustZone.PRESET_DIR, file_name))) * 4

    @classmethod
    def get_validation_schemas_family(cls) -> List[Dict[str, Any]]:
        """Create the validation schema just for supported families.

        :return: List of validation schemas for TZ supported families.
        """
        sch_cfg = ValidationSchemas.get_schema_file(TZ_SCH_FILE)
        sch_cfg["tz_family_rev"]["properties"]["family"]["enum"] = cls.get_supported_families()
        return [sch_cfg["tz_family_rev"]]

    @classmethod
    def get_validation_schemas(cls, family: str, revision: str = "latest") -> List[Dict[str, Any]]:
        """Create the validation schema.

        :param family: Family description.
        :param revision: Chip revision specification, as default, latest is used.
        :raises SPSDKError: Family or revision is not supported.
        :return: List of validation schemas.
        """
        database = cls.load_database()
        sch_cfg = ValidationSchemas.get_schema_file(TZ_SCH_FILE)
        preset_properties = {}

        try:
            revision_info = database.devices.get_by_name(family).revisions.get(revision)
            assert revision_info.data_file
            presets = load_configuration(
                os.path.join(TrustZone.PRESET_DIR, revision_info.data_file)
            )
            for key, value in presets.items():
                preset_properties[key] = {
                    "type": ["string", "number"],
                    "title": "TZ Preset",
                    "description": f"Preset for {key}",
                    "format": "number",
                    "template_value": f"{value}",
                }
            if "patternProperties" in sch_cfg["tz"]["properties"]["trustZonePreset"].keys():
                sch_cfg["tz"]["properties"]["trustZonePreset"].pop("patternProperties")
            sch_cfg["tz"]["properties"]["trustZonePreset"]["properties"] = preset_properties
            sch_cfg["tz_family_rev"]["properties"]["family"]["enum"] = cls.get_supported_families()
            return [sch_cfg["tz_family_rev"], sch_cfg["tz"]]
        except (KeyError, SPSDKError) as exc:
            raise SPSDKError(f"Family {family} or revision {revision} is not supported") from exc

    @classmethod
    def generate_config_template(cls, family: str, revision: str = "latest") -> Dict[str, str]:
        """Generate configuration for selected family.

        :param family: Family description.
        :param revision: Chip revision specification, as default, latest is used.
        :raises SPSDKError: Revision is not supported.
        :return: Dictionary of individual templates (key is name of template, value is template itself).
        """
        ret: Dict[str, str] = {}
        database = cls.load_database()
        if family in cls.get_supported_families():
            try:
                revision_info = database.devices.get_by_name(family).revisions.get(revision)
                revision = revision_info.name
            except (KeyError, SPSDKError) as exc:
                raise SPSDKError(f"Revision {revision} is not supported") from exc

            schemas = cls.get_validation_schemas(family, revision)
            override = {}
            override["family"] = family
            override["revision"] = revision

            yaml_data = ConfigTemplate(
                f"Trust Zone Configuration template for {family}.",
                schemas,
                override,
            ).export_to_yaml()
            ret[f"{family}_tz"] = yaml_data

        return ret

    def __str__(self) -> str:
        return f"<TrustZone: type: {self.type} ({TrustZoneType.desc(self.type)})"

    @classmethod
    def load_database(cls) -> Database:
        """Load data from TZ config file."""
        return Database(cls.CONFIG_FILE)

    @classmethod
    def get_supported_families(cls) -> List[str]:
        """Return list of supported families."""
        return Database.get_devices(cls.CONFIG_FILE).device_names

    def get_families(self) -> List[str]:
        """Return list of supported chip families."""
        return self.database.devices.device_names

    def get_revisions(self, family: Optional[str] = None) -> List[str]:
        """Return a list of revisions for given family."""
        actual_family = family or self.family
        assert actual_family
        revisions = self.database.devices.get_by_name(actual_family).revisions
        return [revision.name for revision in revisions]

    def get_latest_revision(self, family: Optional[str] = None) -> str:
        """Return latest revision for given family."""
        actual_family = family or self.family
        assert actual_family
        return self.database.devices.get_by_name(actual_family).revisions.get_latest().name

    def sanitize_revision(self, family: str, revision: Optional[str] = None) -> str:
        """Sanitize revision.

        if the 'revision' is None return the latest revision
        if the 'revision' is provided return it as lower-case
        """
        if not revision or revision.lower() == "latest":
            return self.get_latest_revision(family)
        return revision.lower()

    def _get_preset_file(self) -> str:
        assert self.family
        file_name = (
            self.database.devices.get_by_name(self.family).revisions.get(self.revision).data_file
        )
        assert file_name, f"Data file for {self.family} does't exist!"
        return os.path.join(TrustZone.PRESET_DIR, file_name)

    def _load_presets(self) -> dict:
        """Load default TrustZone settings for given family and revision."""
        presets = load_configuration(self._get_preset_file())
        # Unify the values format
        return {name: format_value(value_to_int(value), 32) for name, value in presets.items()}

    def _parse_raw_data(self, raw_data: bytes) -> dict:
        """Parse raw data into 'customizations' format."""
        if len(self.presets) > len(raw_data) // 4:
            raise SPSDKError(
                "Trustzone binary file has incorrect raw_data length\n"
                f"Expected: {len(self.presets)}, Got: {len(raw_data) // 4}"
            )

        if len(self.presets) != len(raw_data) // 4:
            logger.warning(
                "Trustzone binary file has incorrect raw_data length\n"
                f"Expected: {len(self.presets)}, Got: {len(raw_data) // 4}"
            )

        registers = struct.unpack(f"<{len(self.presets)}L", raw_data[: len(self.presets) * 4])
        customs = {name: format_value(registers[i], 32) for i, name in enumerate(self.presets)}
        return customs

    @staticmethod
    def validate_custom_data(data: dict, customizations: dict) -> bool:
        """Check whether all register names in custom data are valid (present in presets)."""
        return all(item in data for item in customizations)

    def _custom_export(self) -> bytes:
        if self.presets is None:
            raise SPSDKError("Preset data not present")
        if self.customs is None:
            raise SPSDKError("Data not present")
        modifications = set(self.customs.items()) - set(self.presets.items())
        logger.info(f"{len(modifications)} modifications provided")
        logger.debug(str(modifications).replace(", ", "\n"))
        data = self.presets
        data.update(self.customs)
        registers = [value_to_int(item) for item in data.values()]
        # transform data into binary format (little endian, 32b per register)
        return struct.pack(f"<{len(registers)}I", *registers)

    def export(self) -> bytes:
        """Return the TrustZone data as bytes."""
        return self._custom_export() if self.type == TrustZoneType.CUSTOM else bytes()



#!/usr/bin/env python
# -*- coding: UTF-8 -*-
#
# Copyright 2023 NXP
#
# SPDX-License-Identifier: BSD-3-Clause

"""This module contains HAB related code."""

import logging
from datetime import datetime
from typing import List, Optional

from spsdk.image import segments
from spsdk.image.hab.config_parser import ImageConfig
from spsdk.image.hab.csf_builder import CsfBuildDirector, CsfBuilder
from spsdk.image.hab.hab_binary_image import HabBinaryImage, HabSegment
from spsdk.image.images import BootImgRT
from spsdk.utils.images import BinaryImage
from spsdk.utils.misc import load_binary

logger = logging.getLogger(__name__)


class HabContainer:
    """Hab container."""

    IVT_VERSION = 0x40

    def __init__(self, hab_image: HabBinaryImage) -> None:
        """HAB Constructor.

        :param binary_image: Binary image with required segments.
        """
        self.hab_image = hab_image

    @property
    def ivt_segment(self) -> Optional[bytes]:
        """IVT segment binary."""
        segment = self.hab_image.get_hab_segment(HabSegment.IVT)
        return segment.binary if segment else None

    @property
    def bdt_segment(self) -> Optional[bytes]:
        """BDT segment binary."""
        segment = self.hab_image.get_hab_segment(HabSegment.BDT)
        return segment.binary if segment else None

    @property
    def dcd_segment(self) -> Optional[bytes]:
        """DCD segment binary."""
        segment = self.hab_image.get_hab_segment(HabSegment.DCD)
        return segment.binary if segment else None

    @property
    def xmcd_segment(self) -> Optional[bytes]:
        """XMCD segment binary."""
        segment = self.hab_image.get_hab_segment(HabSegment.XMCD)
        return segment.binary if segment else None

    @property
    def app_segment(self) -> Optional[bytes]:
        """APP segment binary."""
        segment = self.hab_image.get_hab_segment(HabSegment.APP)
        return segment.binary if segment else None

    @property
    def csf_segment(self) -> Optional[bytes]:
        """APP segment binary."""
        segment = self.hab_image.get_hab_segment(HabSegment.CSF)
        return segment.binary if segment else None

    @classmethod
    def load(
        cls,
        image_config: ImageConfig,
        search_paths: Optional[List[str]] = None,
        timestamp: Optional[datetime] = None,
    ) -> "HabContainer":
        """Load the HAB container object from parsed bd_data configuration.

        :param image_config: Image configuration
        :param search_paths: List of paths where to search for the file, defaults to None
        :param timestamp: Signature timestamp
        """
        hab_image = HabBinaryImage()
        # IVT
        ivt = segments.SegIVT2(HabContainer.IVT_VERSION)
        ivt.app_address = image_config.options.entrypoint_address
        ivt.ivt_address = image_config.options.start_address + image_config.options.ivt_offset
        ivt.bdt_address = ivt.ivt_address + ivt.size
        ivt.csf_address = 0
        hab_image.add_hab_segment(HabSegment.IVT, ivt.export())
        ivt_image = hab_image.get_hab_segment(HabSegment.IVT)
        # BDT
        bdt = segments.SegBDT(app_start=image_config.options.start_address)
        hab_image.add_hab_segment(HabSegment.BDT, bdt.export())
        # DCD
        if image_config.options.dcd_file_path is not None:
            dcd_bin = load_binary(image_config.options.dcd_file_path, search_paths=search_paths)
            hab_image.add_hab_segment(HabSegment.DCD, dcd_bin)
            ivt.dcd_address = ivt.ivt_address + HabBinaryImage.DCD_OFFSET
            ivt_image.binary = ivt.export()
        # XMCD
        if image_config.options.xmcd_file_path is not None:
            xmcd_bin = load_binary(image_config.options.xmcd_file_path, search_paths=search_paths)
            hab_image.add_hab_segment(HabSegment.XMCD, xmcd_bin)
        # APP
        app_bin = BinaryImage.load_binary_image(
            image_config.elf_file,
            search_paths=search_paths,
        )
        app_offset = image_config.options.initial_load_size - image_config.options.ivt_offset
        hab_image.add_hab_segment(HabSegment.APP, app_bin.export(), offset_override=app_offset)

        bdt.app_length = image_config.options.ivt_offset + len(hab_image)
        bdt_image = hab_image.get_hab_segment(HabSegment.BDT)
        bdt_image.binary = bdt.export()
        # Calculate CSF offset
        app_image = hab_image.get_hab_segment(HabSegment.APP)
        image_len = app_offset + len(app_image) + image_config.options.ivt_offset
        csf_offset = HabContainer._calculate_csf_offset(image_len)
        csf_offset = csf_offset - image_config.options.ivt_offset

        csf_builder = CsfBuilder(
            image_config,
            csf_offset=csf_offset,
            search_paths=search_paths,
            timestamp=timestamp,
            hab_image=hab_image,
        )
        if csf_builder.is_authenticated or csf_builder.is_encrypted:
            bdt.app_length = image_config.options.ivt_offset + csf_offset + HabBinaryImage.CSF_SIZE
            if csf_builder.is_encrypted:
                bdt.app_length += HabBinaryImage.KEYBLOB_SIZE
            bdt_image.binary = bdt.export()
            ivt.csf_address = ivt.ivt_address + csf_offset
            ivt_image.binary = ivt.export()
        # CSF
        director = CsfBuildDirector(csf_builder)
        director.build_csf()
        return HabContainer(hab_image=hab_image)

    @staticmethod
    def _calculate_csf_offset(image_len: int) -> int:
        """Calculate CSF offset from image length.

        :param image_len: Image length
        :return: CSF offset
        """
        csf_offset = image_len + (16 - (image_len % 16))
        csf_offset = ((csf_offset + 0x1000 - 1) // 0x1000) * 0x1000
        return csf_offset

    @classmethod
    def parse(cls, binary: bytes) -> "HabContainer":
        """Parse existing binary into HAB container object.

        :param binary: Binary to be parsed
        """
        rt_img = BootImgRT.parse(binary)
        # IVT
        hab_image = HabBinaryImage()
        hab_image.add_hab_segment(HabSegment.IVT, rt_img.ivt.export())
        # BDT
        if rt_img.bdt is not None:
            hab_image.add_hab_segment(HabSegment.BDT, rt_img.bdt.export())
        # DCD
        if rt_img.dcd is not None:
            hab_image.add_hab_segment(HabSegment.DCD, rt_img.dcd.export())
        # XMCD
        if rt_img.xmcd is not None:
            hab_image.add_hab_segment(HabSegment.XMCD, rt_img.xmcd.export())
        # CSF
        if rt_img.csf is not None:
            hab_image.add_hab_segment(
                HabSegment.CSF,
                rt_img.csf.export(),
                offset_override=rt_img.ivt.csf_address - rt_img.ivt.ivt_address,
            )
        # APP
        if rt_img.app is not None:
            hab_image.add_hab_segment(
                HabSegment.APP,
                rt_img.app.export(),
                offset_override=rt_img.app_offset - rt_img.ivt_offset,
            )
        return HabContainer(hab_image)

    def export(self) -> bytes:
        """Export into binary."""
        return self.hab_image.export()



#!/usr/bin/env python
# -*- coding: UTF-8 -*-
#
# Copyright 2022-2023 NXP
#
# SPDX-License-Identifier: BSD-3-Clause

"""Module for serial communication with a target device using MBoot protocol."""

import logging
import os
import struct
import time
from io import FileIO
from typing import List, Optional, Tuple, Union

from crcmod.predefined import mkPredefinedCrcFun

from spsdk.mboot.commands import CmdPacket, CmdResponse, parse_cmd_response
from spsdk.mboot.exceptions import McuBootConnectionError, McuBootDataAbortError, McuBootError
from spsdk.utils.easy_enum import Enum
from spsdk.utils.misc import Timeout

from .base import MBootInterface

logger = logging.getLogger(__name__)

########################################################################################################################
# Devices
########################################################################################################################

SDIO_DEVICES = {
    # NAME   | VID   | PID
    "RW61x": (0x0471, 0x0209),
}


def scan_sdio(device_path: Optional[str] = None) -> List["Sdio"]:
    """Scan connected SDIO devices.

    :param device_path: device path string
    :return: matched SDIO device
    """
    if device_path is None:
        logger.debug("No sdio path")
        return []
    try:
        logger.debug(f"Checking path: {device_path}")
        interface = Sdio(path=device_path)
        interface.open()
        interface.close()
        return [interface] if interface else []
    except Exception as e:  # pylint: disable=broad-except
        logger.debug(f"{type(e).__name__}: {e}")
        return []


def calc_crc(data: bytes) -> int:
    """Calculate CRC from the data.

    :param data: data to calculate CRC from
    :return: calculated CRC
    """
    crc_function = mkPredefinedCrcFun("xmodem")
    return crc_function(data)


########################################################################################################################
# SDIO Interface Class
########################################################################################################################
class FPType(Enum):
    """Type of frames used in serial communication."""

    ACK = 0xA1
    NACK = 0xA2
    ABORT = 0xA3
    CMD = 0xA4
    DATA = 0xA5
    PING = 0xA6
    PINGR = 0xA7


class Sdio(MBootInterface):
    """SDIO interface."""

    FRAME_START_BYTE = 0x5A
    FRAME_START_BYTE_NOT_READY = 0x00

    @property
    def name(self) -> str:
        """Get the name of the device.

        :return: Name of the device.
        """
        for name, value in SDIO_DEVICES.items():
            if value[0] == self.vid and value[1] == self.pid:
                return name
        return "Unknown"

    @property
    def is_opened(self) -> bool:
        """Indicates whether device is open.

        :return: True if device is open, False othervise.
        """
        return self.device is not None and self._opened

    def __init__(self, path: Optional[str] = None) -> None:
        """Initialize the SDIO interface object.

        :raises McuBootConnectionError: when the path is empty
        """
        super().__init__()
        self._opened = False
        # Temporarily use hard code until there is a way to retrive VID/PID
        self.vid = 0x0471
        self.pid = 0x0209
        self.timeout = 2000
        if path is None:
            raise McuBootConnectionError("No SDIO device path")
        self.path = path
        self.is_blocking = False
        self.device: Optional[FileIO] = None

    def info(self) -> str:
        """Return information about the SDIO interface."""
        return f"{self.name:s} (0x{self.vid:04X}, 0x{self.pid:04X})"

    def open(self) -> None:
        """Open the interface with non-blocking mode.

        :raises McuBootError: if non-blocking mode is not available
        :raises McuBootConnectionError: if no device path is available
        :raises McuBootConnectionError: if no device is available
        :raises McuBootConnectionError: if the device can not be opened
        """
        if self.is_blocking:
            logger.debug("open with blocking mode")
            self.open_blocking()
            return

        logger.debug("Open Interface with non-blocking mode.")
        if not hasattr(os, "set_blocking"):
            raise McuBootError("Opening in non-blocking mode is available only on Linux")
        if not self._opened:
            try:
                self.device = open(self.path, "rb+", buffering=0)
                if self.device is None:
                    raise McuBootConnectionError("No device available")
                # pylint: disable=no-member     # this is available only on Unix
                os.set_blocking(self.device.fileno(), False)  # type: ignore
                self._opened = True
            except Exception as error:
                raise McuBootConnectionError(
                    f"Unable to open device '{self.path}' VID={self.vid} PID={self.pid}"
                ) from error

    def open_blocking(self) -> None:
        """Open the interface with blocking mode.

        :raises McuBootConnectionError: if no device is available
        :raises McuBootConnectionError: if the device can not be opened
        """
        logger.debug("Open Interface")
        if not self._opened:
            try:
                self.device = open(self.path, "rb+", buffering=0)
                self._opened = True
            except Exception as error:
                raise McuBootConnectionError(
                    f"Unable to open device '{self.path}' VID={self.vid} PID={self.pid}"
                ) from error

    def close(self) -> None:
        """Close the interface.

        :raises McuBootConnectionError: if no device is available
        :raises McuBootConnectionError: if the device can not be opened
        """
        logger.debug("Close Interface")
        if not self.device:
            raise McuBootConnectionError("No device available")
        if self._opened:
            try:
                self.device.close()
                self._opened = False
            except Exception as error:
                raise McuBootConnectionError(
                    f"Unable to close device '{self.path}' VID={self.vid} PID={self.pid}"
                ) from error

    def write(self, packet: Union[CmdPacket, bytes]) -> None:
        """Write data to the SDIO interface; data might be in form of 'CmdPacket' or bytes.

        :param packet: Data to send
        :raises McuBootError: Raises an error if packet type is incorrect
        :raises McuBootConnectionError: Raises an error if device is not opened for writing
        :raises McuBootConnectionError: Raises an error if device is not available
        """
        if not self.device:
            raise McuBootConnectionError("No device available")
        if not self.is_opened:
            raise McuBootConnectionError("Device is opened for writing")

        if isinstance(packet, CmdPacket):
            data = packet.to_bytes(padding=False)
            frame_type = FPType.CMD
        elif isinstance(packet, (bytes, bytearray)):
            data = packet
            frame_type = FPType.DATA
        else:
            raise McuBootError("Packet has to be either 'CmdPacket' or 'bytes'")

        frame = self._create_frame(data, frame_type)
        self._send_frame(frame, wait_for_ack=True)

    def _write(self, data: bytes) -> None:
        """Send data to device with non-blocking mode.

        :param data: Data to send
        :raises McuBootConnectionError: Raises an error if device is not available
        :raises McuBootConnectionError: When sending the data fails
        :raises TimeoutError: When timeout occurs
        """
        if not self.device:
            raise McuBootConnectionError("No device available")

        if self.is_blocking:
            logger.debug("_write with blocking mode")
            self._write_blocking(data)
            return

        logger.debug("_write with non-blocking mode")
        logger.debug(f"[{' '.join(f'{b:02x}' for b in data)}]")
        tx_len = len(data)
        timeout = Timeout(self.timeout)
        while tx_len > 0:
            try:
                wr_count = self.device.write(data)
                time.sleep(0.05)
                data = data[wr_count:]
                tx_len -= wr_count
            except Exception as e:
                raise McuBootConnectionError(str(e)) from e
            if timeout.overflow():
                raise TimeoutError()

    def _write_blocking(self, data: bytes) -> None:
        """Send data to device with blocking mode.

        :param data: Data to send
        :raises McuBootConnectionError: Raises an error if device is not available
        :raises McuBootConnectionError: When sending the data fails
        """
        if not self.device:
            raise McuBootConnectionError("No device available")

        logger.debug(f"[{' '.join(f'{b:02x}' for b in data)}]")
        try:
            self.device.write(data)
        except Exception as e:
            raise McuBootConnectionError(str(e)) from e

    def read(self) -> Union[CmdResponse, bytes]:
        """Read data on the IN endpoint associated to the HID interface.

        :return: Return CmdResponse object.
        :raises McuBootConnectionError: Raises an error if device is not opened for reading
        :raises McuBootConnectionError: Raises if device is not available
        :raises McuBootDataAbortError: Raises if reading fails
        :raises TimeoutError: When timeout occurs
        """
        if not self.is_opened:
            raise McuBootConnectionError("Device is not opened for reading")
        if not self.device:
            raise McuBootConnectionError("Device not available")

        raw_data = self._read(1024)
        if not raw_data:
            logger.error("Cannot read from SDIO device")
            raise TimeoutError()

        _, frame_type = self._parse_frame_header(raw_data)
        length, crc = struct.unpack_from("<HH", raw_data, 2)
        if not length:
            self._send_ack()
            raise McuBootDataAbortError()
        data = raw_data[6 : 6 + length]
        self._send_ack()
        calculated_crc = self._calc_frame_crc(data, frame_type)
        if crc != calculated_crc:
            raise McuBootConnectionError("Received invalid CRC")
        if frame_type == FPType.CMD:
            return parse_cmd_response(data)
        return data

    def _read(self, length: int) -> bytes:
        """Read 'length' amount for bytes from device with non-blocking mode.

        :param length: Number of bytes to read
        :return: Data read from the device
        :raises TimeoutError: When timeout occurs
        :raises McuBootConnectionError: When reading data from device fails
        :raises McuBootConnectionError: Raises if device is not available
        """
        if not self.device:
            raise McuBootConnectionError("Device not available")

        if self.is_blocking:
            logger.debug("_read with blocking mode")
            data = self._read_blocking(length)
            return data

        has_data = 0
        no_data_continuous = 0
        logger.debug("_read with non-blocking mode")
        data = bytearray()
        timeout = Timeout(self.timeout, "ms")
        while len(data) < length:
            try:
                buf = self.device.read(length)
            except Exception as e:
                raise McuBootConnectionError(str(e)) from e

            if buf is None:
                time.sleep(0.05)  # delay for access device
                if has_data != 0:
                    no_data_continuous = no_data_continuous + 1
            else:
                data.extend(buf)
                logger.debug("expend buf")
                has_data = has_data + 1
                no_data_continuous = 0

            if no_data_continuous > 5:
                break
            if timeout.overflow():
                logger.debug("SDIO interface : read timeout")
                break
        if not data:
            raise TimeoutError()
        logger.debug(f"<{' '.join(f'{b:02x}' for b in data)}>")
        return bytes(data)

    def _read_blocking(self, length: int) -> bytes:
        """Read 'length' amount for bytes from device with blocking mode.

        :param length: Number of bytes to read
        :return: Data read from the device
        :raises TimeoutError: When timeout occurs
        :raises McuBootConnectionError: When reading data from device fails
        :raises McuBootConnectionError: Raises if device is not available
        """
        if not self.device:
            raise McuBootConnectionError("Device not available")

        try:
            data = self.device.read(length)
        except Exception as e:
            raise McuBootConnectionError(str(e)) from e
        if not data:
            raise TimeoutError()
        logger.debug(f"<{' '.join(f'{b:02x}' for b in data)}>")
        return data

    def _send_ack(self) -> None:
        ack_frame = struct.pack("<BB", self.FRAME_START_BYTE, FPType.ACK)
        self._send_frame(ack_frame, wait_for_ack=False)

    def _send_frame(self, frame: bytes, wait_for_ack: bool = True) -> None:
        """Send a frame to SDIO.

        :param frame: Data to send
        :param wait_for_ack: Wait for ACK frame from device, defaults to True
        """
        self._write(frame)
        if wait_for_ack:
            data = self._read(2)
            self._parse_frame_header(data, FPType.ACK)

    def _parse_frame_header(
        self, frame: bytes, expected_frame_type: Optional[int] = None
    ) -> Tuple[int, int]:
        """Read frame header and frame type. Return them as tuple of integers.

        :param expected_frame_type: Check if the frame_type is exactly as expected
        :return: Tuple of integers representing frame header and frame type
        :raises McuBootDataAbortError: Target sens Data Abort frame
        :raises McuBootConnectionError: Unexpected frame header or frame type (if specified)
        :raises McuBootConnectionError: When received invalid ACK
        """
        header, frame_type = struct.unpack_from("<BB", frame, 0)
        if header != self.FRAME_START_BYTE:
            raise McuBootConnectionError(
                f"Received invalid frame header '{header:#X}' expected '{self.FRAME_START_BYTE:#X}'"
            )
        if frame_type == FPType.ABORT:
            raise McuBootDataAbortError()
        if expected_frame_type:
            if frame_type != expected_frame_type:
                raise McuBootConnectionError(
                    f"received invalid ACK '{frame_type:#X}' expected '{expected_frame_type:#X}'"
                )
        return header, frame_type

    def _create_frame(self, data: bytes, frame_type: int) -> bytes:
        """Encapsulate data into frame."""
        crc = self._calc_frame_crc(data, frame_type)
        frame = struct.pack(
            f"<BBHH{len(data)}B",
            self.FRAME_START_BYTE,
            frame_type,
            len(data),
            crc,
            *data,
        )
        return frame

    def _calc_frame_crc(self, data: bytes, frame_type: int) -> int:
        """Calculate the CRC of a frame.

        :param data: frame data
        :param frame_type: frame type
        :return: calculated CRC
        """
        crc_data = struct.pack(
            f"<BBH{len(data)}B", self.FRAME_START_BYTE, frame_type, len(data), *data
        )
        return calc_crc(crc_data)



#!/usr/bin/env python
# -*- coding: UTF-8 -*-
#
# Copyright 2021-2022 NXP
#
# SPDX-License-Identifier: BSD-3-Clause

"""Module provides exceptions for TP module."""
from spsdk import SPSDKError


class SPSDKTpError(SPSDKError):
    """General TP error."""


class SPSDKTpConfigError(SPSDKTpError):
    """General TP configuration error."""


class SPSDKTpTimeoutError(SPSDKTpError):
    """TP operation has overflow timeout."""


class SPSDKTpTargetError(SPSDKTpError):
    """TP target failed."""
